/**
  *  \file server/net.cc
  */

#include <sys/types.h>
#include <sys/socket.h>
#include <fcntl.h>
#include <netdb.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include "server/net.h"

SocketProcessor::~SocketProcessor()
{ }

int
SocketProcessor::withAddress(string_t host, string_t port)
{
    /* Where do we want to connect? */
    struct addrinfo hints;
    struct addrinfo* result;
    memset(&hints, 0, sizeof(hints));
    hints.ai_socktype = SOCK_STREAM;
    int ai = getaddrinfo(host.c_str(), port.c_str(), &hints, &result);
    if (ai != 0) {
        fprintf(stderr, "getaddrinfo(%s): %s\n", host.c_str(), gai_strerror(ai));
        return -1;
    }

    /* Found anything? */
    if (result == 0) {
        fprintf(stderr, "getaddrinfo(%s): not found\n", host.c_str());
        return -1;
    }

    /* Make a socket */
    int sock = -1;
    for (struct addrinfo* p = result; p != 0; p = p->ai_next) {
        sock = socket(p->ai_family, p->ai_socktype, p->ai_protocol);
        if (sock >= 0) {
            if (process(sock, p->ai_addr, p->ai_addrlen)) {
                break;
            }
            close(sock);
            sock = -1;
        }
    }
    freeaddrinfo(result);

    /* Return it */
    return sock;
}

/****************************** TimedConnect *****************************/

bool
TimedConnect::process(int sock, const sockaddr* addr, socklen_t len)
{
    fcntl(sock, F_SETFL, O_NONBLOCK);
    if (connect(sock, addr, len) == 0) {
        return true;
    }
    if (errno == EINPROGRESS) {
        fd_set w;
        FD_ZERO(&w);
        FD_SET(sock, &w);
        struct timeval tv;
        tv.tv_sec = secs;
        tv.tv_usec = 0;
        errno = 0;
        if (select(sock+1, 0, &w, 0, &tv) > 0 && FD_ISSET(sock, &w)) {
            /* Connect finished. Did it succeed? */
            int status = 424242;
            socklen_t length = sizeof(status);
            if (getsockopt(sock, SOL_SOCKET, SO_ERROR, &status, &length) == 0 && status == 0) {
                /* Success */
                return true;
            } else {
                return false;
            }
        } else {
            /* Timeout */
            return false;
        }
    } else {
        return false;
    }
}

/******************************** Connect ********************************/

bool
Connect::process(int sock, const sockaddr* addr, socklen_t len)
{
    return (connect(sock, addr, len) == 0);
}

/********************************** Bind *********************************/

bool
Bind::process(int sock, const sockaddr* addr, socklen_t len)
{
    int one = 1;
    setsockopt(sock, SOL_SOCKET, SO_REUSEADDR, &one, sizeof(one));
    return (bind(sock, addr, len) == 0);
}
